#include "common/hardware_info.h"
#include "datafox/communication/module_hardware.h"

#include <string>
#include <bitset>

HardwareInfo::HardwareInfo()
{
    reset();
}

HardwareInfo::~HardwareInfo()
{
}

void HardwareInfo::reset()
{
    // Version der Konfiguration
    m_version = 4;
    m_fileHardwareInfo.reset();
    std::fill( reinterpret_cast<uint8_t *>( &m_hwInfo ), reinterpret_cast<uint8_t *>( &m_hwInfo ) + sizeof( StructHardwareInfo ), 0 );
}

bool HardwareInfo::isMasterIV()
{
    bool res = false;
    ItemDeviceHardwareInfo &deviceHardwareInfo = fileHardwareInfo().rootItemDeviceHardwareInfo();
    // Mainboard an die Konfiguration anfügen.
    ItemBoard::typeChilds &childs = deviceHardwareInfo.childs();
    if ( childs.size() == 0 )
        return res;

    ItemBoard &mainBoard = childs[0];

    // Gerätetyp muss einer dieser sein.
    switch ( deviceHardwareInfo.hcn().device )
    {
    case IV_VALUE_TYPE_DEVICE_PZE:
    case IV_VALUE_TYPE_DEVICE_ZK:
    case IV_VALUE_TYPE_DEVICE_BDE:
    case IV_VALUE_TYPE_DEVICE_MOBIL:
    case IV_VALUE_TYPE_DEVICE_MDE:
    case IV_VALUE_TYPE_DEVICE_AE:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY:
    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:
    case IV_VALUE_TYPE_DEVICE_FLEX:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:
        // Zusätzlich muss die Boardversion kleiner 4.0 sein.
        if ( mainBoard.hc().revision < 40 )
            res = true;
    }

    return res;
}

bool HardwareInfo::isMasterIV( const StructHardwareInfo &info )
{
    bool res = false;

    // Gerätetyp muss einer dieser sein.
    switch ( info.typeDevice )
    {
    case IV_VALUE_TYPE_DEVICE_PZE:
    case IV_VALUE_TYPE_DEVICE_ZK:
    case IV_VALUE_TYPE_DEVICE_BDE:
    case IV_VALUE_TYPE_DEVICE_MOBIL:
    case IV_VALUE_TYPE_DEVICE_MDE:
    case IV_VALUE_TYPE_DEVICE_AE:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY:
    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:
    case IV_VALUE_TYPE_DEVICE_FLEX:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:
        // Zusätzlich muss die Boardversion kleiner 4.0 sein.
        if ( info.typeBoard < 0x40 )
            res = true;
    }

    return res;
}

bool HardwareInfo::importStructHardwareInfo( const StructHardwareInfo &info )
{
    std::string name;
    std::string article;

    // Klassenmember zurücksetzen.
    reset();

    // Die HWConfig-Version uebernehmen.
    setVersion( info.version );

    // Fuer spaeteren Abruf merken.
    m_hwInfo = info;

    ItemDeviceHardwareInfo &deviceHardwareInfo = fileHardwareInfo().rootItemDeviceHardwareInfo();
    ItemBoard::typeChilds &childs = deviceHardwareInfo.childs();
    childs.resize( 1 ); // Eintrag für das Mainboard.
    // Mainboard an die Konfiguration anfügen.
    ItemBoard &mainBoard = childs[0];

    // Daten der Headerklasse eintragen
    deviceHardwareInfo.hcn().version = 1;
    // Gerätetype übernehmen
    switch ( info.typeDevice )
    {
    case IV_VALUE_TYPE_DEVICE_PZE:
    case IV_VALUE_TYPE_DEVICE_ZK:
    case IV_VALUE_TYPE_DEVICE_BDE:
    case IV_VALUE_TYPE_DEVICE_MOBIL:
    case IV_VALUE_TYPE_DEVICE_MDE:
    case IV_VALUE_TYPE_DEVICE_AE:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY:
    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:
    case IV_VALUE_TYPE_DEVICE_FLEX:
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:
        deviceHardwareInfo.hcn().device = info.typeDevice;
        break;

    default: // Alle anderen Gerätetypen entsprechen keinem MasterIV.
        return false;
    }

    // Daten der Hauptplatine eintragen.
    mainBoard.hc().number = 0;
    article = "---";
    if ( info.typeDevice == IV_VALUE_TYPE_DEVICE_BDE )
    {
        mainBoard.hc().id = 50014; // BDE-MasterIV Mainboard
        name = "BDE-MasterIV Mainboard";
    }
    else if ( info.typeDevice == IV_VALUE_TYPE_DEVICE_TIMEBOY || info.typeDevice == IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE )
    {
        mainBoard.hc().id = 50015; // TimeboyIV Mainboard
        name = "TimeboyIV Mainboard";
    }
    else
    {
        mainBoard.hc().id = 50016; // KombiV1-3 Mainboard
        name = "Kombi V1-3 Mainboard";
    }
    // Boardrevision übernehmen
    mainBoard.hc().revision = info.typeBoard / 16 * 10 + info.typeBoard % 16;
    // Stücklistenindex
    mainBoard.hc().index = 1;
    // usw..
    
    article.copy( mainBoard.hc().article, sizeof mainBoard.hc().article );
    mainBoard.hc().type = Mainboard;
    name.copy( mainBoard.hc().name, sizeof mainBoard.hc().name );
    
    // Folgende Platznummern sind den Typen und Komponenten zugewiesen.
    //  1 => TCPIP
    //       100001 LAN-XPort
    //       100002 WLAN-MatchPort
    //       105002 WLAN RS9110-N-11-24
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeTCPIP != IV_VALUE_TYPE_TCPIP_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 1;
        article = "---";
        switch ( info.typeTCPIP )
        {
        case IV_VALUE_TYPE_TCPIP_XPORT:
            board.hc().id = 100001;
            name = "LAN-XPort";
            break;

        case IV_VALUE_TYPE_TCPIP_MATCHPORT:
            board.hc().id = 100002;
            name = "WLAN-MatchPort";
            break;

        case IV_VALUE_TYPE_TCPIP_RS9110_N_11_24:
            board.hc().id = 105002;
            name = "WLAN RS9110-N-11-24";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }


    // ################################
    //  2 => Mobilfunk
    //       100003 Mobile MC35-39
    //       105001 Mobile MC55i
    //       105002 WLAN RS9110-N-11-24
    //       105003 BG96 Quectel
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeModemMC != IV_VALUE_TYPE_MODEM_MC_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 2;
        article = "---";
        switch ( info.typeModemMC )
        {
        case IV_VALUE_TYPE_MODEM_MC_39:
            board.hc().id = 100003;
            name = "Mobile MC35-39";
            break;

        case IV_VALUE_TYPE_MODEM_MC_55:
            board.hc().id = 105001;
            name = "Mobile MC55i";
            break;

        case IV_VALUE_TYPE_MODEM_RS9110_N_11_24: // Verbautes WLAN-Modul auf dem Platz für GSM-Modems.
            board.hc().id = 105002;
            name = "WLAN RS9110-N-11-24";
            break;

        case IV_VALUE_TYPE_MODEM_BG96:
            board.hc().id = 105003;
            name = "Mobile BG96";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  3 => RS485
    //       100000 RS485
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentRS485_0 )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 3;
        article = "---";
        board.hc().id = 100000;
        name = "RS485";
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  4 => USB
    //       100004 USB
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentUSB )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 4;
        article = "---";
        board.hc().id = 100004;
        name = "USB";
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  5 => Bluetooth
    //       100005 Bluegiga Wrap Thor
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeRadio != IV_VALUE_TYPE_RADIO_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 5;
        article = "---";
        switch ( info.typeRadio )
        {
        case IV_VALUE_TYPE_RADIO_WRAPTHOR:
            board.hc().id = 100005;
            name = "Bluegiga Wrap Thor";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  6 => Zutrittskontrolle
    //       100006 RS485-Access control
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentRS485_1 )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 6;
        article = "---";
        board.hc().id = 100006;
        name = "RS485-Access control";
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  7 => Transponderleser
    //       102001 TSR30 Reader 125kHz
    //       102002 TSR32 Reader 125kHz
    //       102003 Primo 110 Legic Reader
    //       102004 Mifare Classic Reader
    //       102005 Smartrelais G1
    //       102006 Nedap Reader XS70
    //       102007 iButton
    //       102008 HID Proxy Reader 125kHz
    //       102009 TWN3 Multi ISO Reader
    //       102010 Primo 130 Legic Reader
    //       102011 TSR21 Reader 125kHz
    //       102012 HID iClass Reader
    //       102013 Multi ISO Reader Timeboy
    //       102014 Legic Reader Timeboy
    //       102015 Smartrelais G2
    //       102016 TWN3 Multi NFC Reader
    //       102017 UHF Reader 500 mW
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeTransponder != IV_VALUE_TYPE_TRANSPONDER_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 7;
        article = "---";
        switch ( info.typeTransponder )
        {
        case IV_VALUE_TYPE_TRANSPONDER_TSR30:
            board.hc().id = 102001;
            name = "TSR30 Reader 125kHz";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TSR32:
            board.hc().id = 102002;
            name = "TSR32 Reader 125kHz";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_PRIMO100:
            board.hc().id = 102003;
            name = "Primo 110 Legic Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_MIFARE:
            board.hc().id = 102004;
            name = "Mifare Classic Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS:
            board.hc().id = 102005;
            name = "Smartrelais G1";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_NEDAP:
            board.hc().id = 102006;
            name = "Nedap Reader XS70";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN:
        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN:
            board.hc().id = 102007;
            name = "iButton";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_HID_PROXKEY:
            board.hc().id = 102008;
            name = "HID Proxy Reader 125kHz";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MULTI_ISO:
            board.hc().id = 102009;
            name = "TWN3 Multi ISO Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_PRIMO130:
            board.hc().id = 102010;
            name = "Primo 130 Legic Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TSR21:
            board.hc().id = 102011;
            name = "TSR21 Reader 125kHz";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_HID_ICLASS:
            board.hc().id = 102012;
            name = "HID iClass Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TB_MIFARE:
            board.hc().id = 102013;
            name = "Multi ISO Reader Timeboy";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TB_LEGIC:
            board.hc().id = 102014;
            name = "Legic Reader Timeboy";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS_G2:
            board.hc().id = 102015;
            name = "Smartrelais G2";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MIFARE_NFC:
            board.hc().id = 102016;
            name = "TWN3 Multi NFC Reader";
            break;

        case IV_VALUE_TYPE_TRANSPONDER_IDTRONIC_UHF:
            board.hc().id = 102017;
            name = "UHF Reader 500 mW";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  8 => Display
    //       110001 Monocrome Display 3,5"
    //       100007 6 Zoll Monochrom
    //       110002 Color TFT 1,9" 220x176
    //       110005 Color TFT 2,0" 220x176
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeDisplay != IV_VALUE_TYPE_DISPLAY_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 8;
        article = "---";
        switch ( info.typeDisplay )
        {
        case IV_VALUE_TYPE_DISPLAY_SMALL:
            board.hc().id = 110001;
            name = "Monocrome Display 3,5\"";
            break;

        case IV_VALUE_TYPE_DISPLAY_BIG:
            board.hc().id = 100007;
            name = "6 Zoll Monochrom";
            break;

         case IV_VALUE_TYPE_DISPLAY_SMALLER:
            board.hc().id = 110002;
            name = "Color TFT 1,9\"";
            break;

         case IV_VALUE_TYPE_DISPLAY_TIMEBOY_2ZOLL:
             board.hc().id = 110005;
             name = "Color TFT 2,0\"";
             break;

         default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    //  9 => Tastatur
    //       101002 Keypad PZE Master
    //       101006 Keypad BDE-MasterIV
    //       101003 Keypad AE Master
    //       101007 Keypad PZE-BasicIV
    //       101008 Keypad TimeboyIV
    //       101009 Keypad Flex-MasterIV
    //       101010 Keypad Timeboy-Mobil-PZE
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeKeypad != IV_VALUE_TYPE_KEYPAD_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 9;
        article = "---";
        switch ( info.typeKeypad )
        {
        case IV_VALUE_TYPE_KEYPAD_9:
            board.hc().id = 101002;
            name = "Keypad PZE Master";
            break;

        case IV_VALUE_TYPE_KEYPAD_58:
            board.hc().id = 101006;
            name = "Keypad BDE-MasterIV";
            break;

        case IV_VALUE_TYPE_KEYPAD_29:
            board.hc().id = 101003;
            name = "Keypad AE Master";
            break;

        case IV_VALUE_TYPE_KEYPAD_7:
            board.hc().id = 101007;
            name = "Keypad PZE-BasicIV";
            break;

        case IV_VALUE_TYPE_KEYPAD_25:
            board.hc().id = 101008;
            name = "Keypad TimeboyIV";
            break;

        case IV_VALUE_TYPE_KEYPAD_20:
            board.hc().id = 101009;
            name = "Keypad Flex-MasterIV";
            break;

        case IV_VALUE_TYPE_KEYPAD_3:
            board.hc().id = 101010;
            name = "Keypad Timeboy-Mobil-PZE";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 10 => Multimediacard
    //       100008 Memory extension
    //       100009 Removable data media
    //       100010 USB-Host
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeMMC != IV_VALUE_TYPE_MMC_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 10;
        article = "---";
        switch ( info.typeMMC )
        {
        case IV_VALUE_TYPE_MMC_EXTENSION:
            board.hc().id = 100008;
            name = "Speichererweiterung";
            break;

        case IV_VALUE_TYPE_MMC_DISK:
            board.hc().id = 100009;
            name = "Wechseldatenträger";
            break;

        case IV_VALUE_TYPE_MMC_USB:
            board.hc().id = 100010;
            name = "USB-Host";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 11 => GPS
    //       100011 Furuno GN-80
    //       100012 Lea5H
    //       100013 Navimouse Lea5H
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeGPS != IV_VALUE_TYPE_GPS_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 11;
        article = "---";
        switch ( info.typeGPS )
        {
        case IV_VALUE_TYPE_GPS_GN80:
            board.hc().id = 100011;
            name = "Furuno GN-80";
            break;

        case IV_VALUE_TYPE_GPS_5H:
            board.hc().id = 100012;
            name = "Lea5H";
            break;

        case IV_VALUE_TYPE_GPS_MOUSE_5H:
            board.hc().id = 100013;
            name = "Navimouse Lea5H";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 12 => Barcodeleser
    //       100014 Barcode intern
    //       100015 Barcode Zebex
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeDCF77 != IV_VALUE_TYPE_DCF77_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 12;
        article = "---";
        switch ( info.typeDCF77 )
        {
        case IV_VALUE_TYPE_DCF77_BARCODE:
            board.hc().id = 100014;
            name = "Barcode internal";
            break;

        case IV_VALUE_TYPE_DCF77_BARCODE_ZEBEX:
            board.hc().id = 100015;
            name = "Barcode Zebex";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 13 => Analoge Eingänge
    //       100016 Analog input
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentInAnalog )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 13;
        article = "---";
        board.hc().id = 100016;
        name = "Analog input";
        // Boardrevision ist hier die Anzahl der Eingänge.
        std::bitset<sizeof(info.componentInAnalog) * 8> b( info.componentInAnalog );
        int pos;
        for ( pos = 0; b.test( pos ); ++pos );
        if ( (std::size_t)pos != b.count() )
            return false;

        board.hc().revision = static_cast<uint8_t>( b.count() );
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 14 => Digitale Eingänge
    //       100017 Digital input
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentInDigital )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 14;
        article = "---";
        board.hc().id = 100017;
        name = "Digital input";
        // Boardrevision ist hier die Anzahl der Eingänge.
        std::bitset<sizeof(info.componentInDigital) * 8> b( info.componentInDigital );
        int pos;
        for ( pos = 0; b.test( pos ); ++pos );
        if ( (std::size_t)pos != b.count() )
            return false;

        board.hc().revision = static_cast<uint8_t>( b.count() );
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 15 => Digitale Ausgänge
    //       100018 Digital output
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentOutDigital )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 15;
        article = "---";
        board.hc().id = 100018;
        name = "Digital output";
        // Boardrevision ist hier die Anzahl der Ausgänge.
        std::bitset<sizeof(info.componentOutDigital) * 8> b( info.componentOutDigital );
        int pos;
        for ( pos = 0; b.test( pos ); ++pos );
        if ( (std::size_t)pos != b.count() )
            return false;

        board.hc().revision = static_cast<uint8_t>( b.count() );
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 16 => Fingerprint
    //       104000 Fingerprint Biokey3000
    //       104001 Fingerprint Biokey4000
    //       104002 Fingerprint Biokey4020
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeFingerprint != IV_VALUE_TYPE_FINGERPRINT_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 16;
        article = "---";
        switch ( info.typeFingerprint )
        {
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000:
            board.hc().id = 104000;
            name = "Fingerprint Biokey3000";
            break;

        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4000:
            board.hc().id = 104001;
            name = "Fingerprint Biokey4000";
            break;

        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4020:
            board.hc().id = 104002;
            name = "Fingerprint Biokey4020";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 17 => Touchpanel
    //       100019 4 Zoll touchpanel
    //       100020 6 Zoll touchpanel
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.typeTouch != IV_VALUE_TYPE_TOUCH_NONE )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 17;
        article = "---";
        switch ( info.typeTouch )
        {
        case IV_VALUE_TYPE_TOUCH_SMALL:
            board.hc().id = 100019;
            name = "4 Zoll touchpanel";
            break;

        case IV_VALUE_TYPE_TOUCH_BIG:
            board.hc().id = 100020;
            name = "6 Zoll touchpanel";
            break;

        default:
            return false;
        }
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 18 => Lithium Batterie
    //       100021 Lithium battery
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentBattery )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 18;
        article = "---";
        board.hc().id = 100021;
        name = "Lithium battery";
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // ################################
    // 19 => Ladeschaltung
    //       100022 Charging circuit
    // Wenn Belegt, dann die Daten für den Platz eintragen.
    if ( info.componentCharge )
    {
        ItemBoard::typeChilds &childs = mainBoard.childs();
        // Neuen Eintrag hinzufügen.
        childs.resize( childs.size() + 1 );
        ItemBoard &board = childs.back();
        board.hc().number = 19;
        article = "---";
        board.hc().id = 100022;
        name = "Charging circuit";
        // Boardrevision ist immer 1.0, Bestücklistenindex a = 1
        board.hc().revision = 10;
        board.hc().index = 1;
        // weitere Angaben in die Struktur übernehmen.
        article.copy( board.hc().article, sizeof board.hc().article );
        board.hc().type = SubModule;
        name.copy( board.hc().name, sizeof board.hc().name );
    }

    // Durch speichern ohne Dateiangabe in den Puffer sichern, crc-Berechnung etc.
    if ( !fileHardwareInfo().save( "" ) )
        return false;

    return true;
}

StructHardwareInfo HardwareInfo::exportStructHardwareInfo()
{
    StructHardwareInfo hwInfo;
    if ( version() < 4 )
    {
        hwInfo = importedStructHardwareInfo();
    }
    else
    {
        // Aktuell werden nur die Grundangaben uebernommen.
        std::fill( reinterpret_cast<uint8_t *>( &hwInfo ), reinterpret_cast<uint8_t *>( &hwInfo ) + sizeof( StructHardwareInfo ), 0 );
        hwInfo.version = version();
        hwInfo.typeDevice = deviceTypeOld();
        hwInfo.typeBoard = mainBoardRevision() / 10 * 16 + mainBoardRevision() % 10;
    }
        
    return hwInfo;
}

StructHardwareInfo HardwareInfo::importedStructHardwareInfo()
{
    if ( version() >= 4 ) // Es handelt sich aktuell um keine importierte Struktur.
    {
        std::fill( reinterpret_cast<uint8_t *>( &m_hwInfo ), reinterpret_cast<uint8_t *>( &m_hwInfo ) + sizeof( StructHardwareInfo ), 0 );
    }
        
    return m_hwInfo;
}

void HardwareInfo::setFileHardwareInfo( const FileHardwareInfo &file )
{
    m_fileHardwareInfo = file;
}

FileHardwareInfo &HardwareInfo::fileHardwareInfo()
{
    return m_fileHardwareInfo;
}

void HardwareInfo::setVersion( uint8_t version )
{
    m_version = version;
}

uint8_t HardwareInfo::version()
{
    return m_version;
}

std::string HardwareInfo::deviceName(EnumDeviceType type)
{
    std::string temp;
    switch ( type )
    {          
    case PZEMasterIV:        temp = "PZE-MasterIV"; break;
    case ZKMasterIV:         temp = "ZK-MasterIV"; break;
    case BDEMasterIV:        temp = "BDE-MasterIV"; break;
    case MobilMasterIV:      temp = "Mobil-MasterIV"; break;
    case MDEBoxIV:           temp = "MDE-BoxIV"; break;
    case AEMasterIV:         temp = "AE-MasterIV"; break;
    case TimeboyIV:          temp = "TimeboyIV"; break;
    case ExklusivLineIV:     temp = "Exklusiv-Line IV"; break;
    case FlexMasterIV:       temp = "Flex-MasterIV"; break;
    case TimeboyMobilPZE:    temp = "TimeboyIV Mobil-PZE"; break;
    case IPCExtensionA:      temp = "IPC Embedded-System A"; break;
    case EVO_2_8_Pure:       temp = "EVO 2.8"; break;
    case EVO_4_3:            temp = "EVO 4.3"; break;
    case IPCExtensionB:      temp = "IPC Embedded-System B"; break;
    case IPCExtensionC:      temp = "IPC Embedded-System C"; break;
    case KYO_Inloc:          temp = "KYO Inloc"; break;
    case IOBox:              temp = "IO-Box V4"; break;
    case MobilBox:           temp = "Mobil-Box V4"; break;
    case TimeboyTBDock:      temp = "Dockingstation V2"; break;
    case MobileTBDock:       temp = "Fahrzeugdatenlogger V2"; break;
    case KYO_Oneloc:         temp = "KYO Oneloc"; break;
    case EVO_3_5_Universal:  temp = "EVO 3.5 Universal"; break;
    case SAMS_OCL_Reader:    temp = "SAMS-OCL-Reader"; break;
    case EVO_3_5_Pure:       temp = "EVO 3.5"; break;
    case EVO_Agera:          temp = "EVO Agera"; break;
    case KYO_Cenloc:         temp = "KYO Cenloc"; break;
    case Vario_10:           temp = "Vario 10"; break;
    case Vario_12:           temp = "Vario 12"; break;
    case Vario_15:           temp = "Vario 15"; break;
    case Vario_17:           temp = "Vario 17"; break;
    case IPC_EVO_12_1:       temp = "IPC EVO 12.1"; break;
    case IPC_EVO_15:         temp = "IPC EVO 15"; break;
    case IPC_EVO_18_5:       temp = "IPC EVO 18.5"; break;
    case IPC_EVO_24:         temp = "IPC EVO 24"; break;
    case EVO_Intera_2:       temp = "EVO Intera 2"; break;
    case EVO_4_6_FlexKey:    temp = "EVO 4.6 FlexKey"; break;
    case EVO_5_0_Pure:       temp = "EVO 5.0 Pure"; break;
        // !!! Auch immer in der ItemDeviceHardwareInfo::toJson nachpflegen !!!
    }

    return temp;
}

HardwareInfo::EnumDeviceType HardwareInfo::deviceType()
{
    return static_cast<EnumDeviceType>( fileHardwareInfo().rootItemDeviceHardwareInfo().hcn().device );
}

// Unterschiede in dem Typemapping der Vorgaengerversion und aktuellen ausgleichen.
HardwareInfo::EnumDeviceType HardwareInfo::deviceTypeOld()
{
    EnumDeviceType type = deviceType();
    switch ( type )
    {
    case PZEMasterIV:
    case ZKMasterIV:
    case BDEMasterIV:
    case MobilMasterIV:
    case MDEBoxIV:
    case AEMasterIV:
    case TimeboyIV:
    case ExklusivLineIV:
    case FlexMasterIV:
    case TimeboyMobilPZE:
        break;
    case IPCExtensionA:      type = EVO_4_3; break; // Die Neue HIP vergibt eine 
    case EVO_2_8_Pure:       type = IPCExtensionA; break;
    case EVO_4_3:            type = EVO_2_8_Pure; break;
    case IPCExtensionB:
    case IPCExtensionC:
        break;
    case KYO_Inloc:          type = IOBox; break;
    case IOBox:              type = KYO_Inloc; break;
    case MobilBox:
    case TimeboyTBDock:
    case MobileTBDock:
    case KYO_Oneloc:
    case EVO_3_5_Universal:
    case SAMS_OCL_Reader:
    case EVO_3_5_Pure:
    case EVO_Agera:
    case KYO_Cenloc:
    case Vario_10:
    case Vario_12:
    case Vario_15:
    case Vario_17:
    case IPC_EVO_12_1:
    case IPC_EVO_15:
    case IPC_EVO_18_5:
    case IPC_EVO_24:
    case EVO_Intera_2:
    case EVO_4_6_FlexKey:
    case EVO_5_0_Pure:
        // !!! Auch immer in der ItemDeviceHardwareInfo::toJson nachpflegen !!!
        break;

    }

    return type;
}

uint8_t HardwareInfo::mainBoardRevision()
{
    uint8_t res = 0;
    ItemDeviceHardwareInfo &deviceHardwareInfo = fileHardwareInfo().rootItemDeviceHardwareInfo();
    ItemBoard::typeChilds &childs = deviceHardwareInfo.childs();
    if ( childs.size() > 0 )
    {    
        // Mainboard an die Konfiguration anfügen.
        ItemBoard &mainBoard = childs[0];
        res = mainBoard.hc().revision;
    }

    return res;
}
listItemBoard HardwareInfo::collectModuleIds( const setModuleId &ids )
{
    return fileHardwareInfo().collectModuleIds( ids );
}

listItemBoard HardwareInfo::moduleRS232Communication( const setModuleRS232Communication &module )
{
    return fileHardwareInfo().moduleRS232Communication( module );
}

listItemBoard HardwareInfo::moduleRS485Communication( const setModuleRS485Communication &module )
{
    return fileHardwareInfo().moduleRS485Communication( module );
}

listItemBoard HardwareInfo::onBoardRS485Communication( const setOnBoardRS485Communication &boards )
{
    return fileHardwareInfo().onBoardRS485Communication( boards );
}

listItemBoard HardwareInfo::moduleUSBCommunication( const setModuleUSBCommunication &module )
{
    return fileHardwareInfo().moduleUSBCommunication( module );
}

listItemBoard HardwareInfo::moduleLANCommunication( const setModuleLANCommunication &module )
{
    return fileHardwareInfo().moduleLANCommunication( module );
}

listItemBoard HardwareInfo::onBoardLANCommunication( const setOnBoardLANCommunication &boards )
{
    return fileHardwareInfo().onBoardLANCommunication( boards );
}

listItemBoard HardwareInfo::moduleWLANCommunication( const setModuleWLANCommunication &module )
{
    return fileHardwareInfo().moduleWLANCommunication( module );
}

listItemBoard HardwareInfo::moduleMobileCommunication( const setModuleMobileCommunication &module )
{
    return fileHardwareInfo().moduleMobileCommunication( module );
}

listItemBoard HardwareInfo::moduleBluetoothCommunication( const setModuleBluetoothCommunication &module )
{
    return fileHardwareInfo().moduleBluetoothCommunication( module );
}

listItemBoard HardwareInfo::moduleMMCCommunication( const setModuleMMCCommunication &module )
{
    return fileHardwareInfo().moduleMMCCommunication( module );
}

listItemBoard HardwareInfo::moduleUSBHostCommunication( const setModuleUSBHostCommunication &module )
{
    return fileHardwareInfo().moduleUSBHostCommunication( module );
}

listItemBoard HardwareInfo::moduleAccessControl( const setModuleAccessControl &module )
{
    return fileHardwareInfo().moduleAccessControl( module );
}

listItemBoard HardwareInfo::onBoardAccessControl( const setOnBoardAccessControl &boards )
{
    return fileHardwareInfo().onBoardAccessControl( boards );
}

listItemBoard HardwareInfo::moduleBiokey( const setModuleBiokey &module )
{
    return fileHardwareInfo().moduleBiokey( module );
}

listItemBoard HardwareInfo::moduleGPS( const setModuleGPS &module )
{
    return fileHardwareInfo().moduleGPS( module );
}

listItemBoard HardwareInfo::moduleDisplay( const setModuleDisplay &module )
{
    return fileHardwareInfo().moduleDisplay( module );
}

listItemBoard HardwareInfo::moduleTouch( const setModuleTouch &module )
{
    return fileHardwareInfo().moduleTouch( module );
}

listItemBoard HardwareInfo::moduleTemperatureSensor( const setModuleTemperatureSensor &module )
{
    return fileHardwareInfo().moduleTemperatureSensor( module );
}
